@@ -16,12 +16,14 @@ from abstract_plot_data import AbstractPlotData
 from array_data_source import ArrayDataSource
 from array_plot_data import ArrayPlotData
 from base_xy_plot import BaseXYPlot
+from candle_plot import CandlePlot
 from colormapped_scatterplot import ColormappedScatterPlot
 from contour_line_plot import ContourLinePlot
 from contour_poly_plot import ContourPolyPlot
 from cmap_image_plot import CMapImagePlot
 from data_range_1d import DataRange1D
 from data_view import DataView
+from default_colormaps import Spectral
 from grid_data_source import GridDataSource
 from grid_mapper import GridMapper
 from image_data import ImageData
@@ -94,9 +96,9 @@ class Plot(DataView):
                         "pink", "darkgray", "silver"])
 
     # index into auto_colors list
-    _auto_color_idx = Int(0)
-    _auto_edge_color_idx = Int(0)
-    _auto_face_color_idx = Int(0)
+    _auto_color_idx = Int(-1)
+    _auto_edge_color_idx = Int(-1)
+    _auto_face_color_idx = Int(-1)
 
     # Mapping of renderer type string to renderer class
     # This can be overriden to customize what renderer type the Plot
@@ -108,21 +110,22 @@ class Plot(DataView):
                              img_plot = ImagePlot,
                              cmap_img_plot = CMapImagePlot,
                              contour_line_plot = ContourLinePlot,
-                             contour_poly_plot = ContourPolyPlot))
+                             contour_poly_plot = ContourPolyPlot,
+                             candle = CandlePlot))
 
     #------------------------------------------------------------------------
     # Annotations and decorations
     #------------------------------------------------------------------------
 
     # The title of the plot.
-    title = Property
+    title = Property()
 
     # The font to use for the title.
-    title_font = Str("swiss 16")
+    title_font = Property()
 
     # Convenience attribute for title.overlay_position; can be "top",
     # "bottom", "left", or "right".
-    title_position = Property
+    title_position = Property()
 
     # Use delegates to expose the other PlotLabel attributes of the plot title
     title_text = Delegate("_title", prefix="text", modify=True)
@@ -158,7 +161,7 @@ class Plot(DataView):
                                   "of type " + str(type(data))
 
         if not self._title:
-            self._title = PlotLabel(font=self.title_font, visible=False,
+            self._title = PlotLabel(font="swiss 16", visible=False,
                                    overlay_position="top", component=self)
         if title is not None:
             self.title = title
@@ -378,8 +381,9 @@ class Plot(DataView):
         If *data* has shape (N, M, 3) or (N, M, 4), then it is treated as RGB or
         RGBA (respectively) and *colormap* is ignored.
 
-        If *data* is an array of floating-point data, and no colormap is provided,
-        then a ValueError is thrown.
+        If *data* is an array of floating-point data, then a colormap can
+        be provided via the *colormap* argument, or the default of 'Spectral'
+        will be used.
 
         *Data* should be in row-major order, so that xbounds corresponds to
         *data*'s second axis, and ybounds corresponds to the first axis.
@@ -416,7 +420,7 @@ class Plot(DataView):
         else:
             if colormap is None:
                 if self.color_mapper is None:
-                    raise ValueError("Scalar 2D data requires a colormap.")
+                    colormap = Spectral(DataRange1D(value))
                 else:
                     colormap = self.color_mapper
             elif isinstance(colormap, AbstractColormap):
@@ -496,7 +500,8 @@ class Plot(DataView):
             floating point data.
         type : comma-delimited string of "line", "poly"
             The type of contour plot to add. If the value is "poly"
-            and no colormap is provided, then a ValueError is thrown.
+            and no colormap is provided via the *poly_cmap* argument, then 
+            a default colormap of 'Spectral' is used.
         name : string
             The name of the plot; if omitted, then a name is generated.
         poly_cmap : string
@@ -536,7 +541,7 @@ class Plot(DataView):
                     cmap.range = DataRange1D(value)
         elif type == "poly":
             if poly_cmap is None:
-                raise ValueError("Scalar 2D data requires a colormap.")
+                poly_cmap = Spectral(DataRange1D(value))
             elif isinstance(poly_cmap, FunctionType):
                 poly_cmap = poly_cmap(DataRange1D(value))
             elif getattr(poly_cmap, 'range', 'dummy') is None:
@@ -602,7 +607,133 @@ class Plot(DataView):
         self.plots[name] = [plot]
         return self.plots[name]
 
+    def candle_plot(self, data, name=None, value_scale="linear", origin=None,
+                    **styles):
+        """ Adds a new sub-plot using the given data and plot style.
 
+        Parameters
+        ==========
+        data : list(string), tuple(string)
+            The names of the data to be plotted in the ArrayDataSource.  The
+            number of arguments determines how they are interpreted:
+
+            (index, bar_min, bar_max)
+                filled or outline-only bar extending from **bar_min** to 
+                **bar_max**
+
+            (index, bar_min, center, bar_max)
+                above, plus a center line of a different color at **center**
+
+            (index, min, bar_min, bar_max, max)
+                bar extending from **bar_min** to **bar_max**, with thin 
+                bars at **min** and **max** connected to the bar by a long
+                stem
+
+            (index, min, bar_min, center, bar_max, max)
+                like above, plus a center line of a different color and 
+                configurable thickness at **center**
+
+        name : string
+            The name of the plot.  If None, then a default one is created.
+
+        value_scale : string
+            The type of scale to use for the value axis.  If not "linear",
+            then a log scale is used.
+
+        Styles
+        ======
+        These are all optional keyword arguments.
+        
+        bar_color : string, 3- or 4-tuple
+            The fill color of the bar; defaults to "auto".
+        bar_line_color : string, 3- or 4-tuple
+            The color of the rectangular box forming the bar.
+        stem_color : string, 3- or 4-tuple (default = bar_line_color)
+            The color of the stems reaching from the bar to the min and
+            max values.
+        center_color : string, 3- or 4-tuple (default = bar_line_color)
+            The color of the line drawn across the bar at the center values.
+        line_width : int (default = 1)
+            The thickness, in pixels, of the outline around the bar.
+        stem_width : int (default = line_width)
+            The thickness, in pixels, of the stem lines
+        center_width : int (default = line_width)
+            The width, in pixels, of the line drawn across the bar at the
+            center values.
+        end_cap : bool (default = True)
+            Whether or not to draw bars at the min and max extents of the
+            error bar.
+
+        Returns
+        =======
+        [renderers] -> list of renderers created in response to this call.
+        """
+        if len(data) == 0:
+            return
+        self.value_scale = value_scale
+        
+        if name is None:
+            name = self._make_new_plot_name()
+        if origin is None:
+            origin = self.default_origin
+        plot_name = name
+
+        # Create the datasources
+        if len(data) == 3:
+            index, bar_min, bar_max = map(self._get_or_create_datasource, data)
+            self.value_range.add(bar_min, bar_max)
+            center = None
+            min = None
+            max = None
+        elif len(data) == 4:
+            index, bar_min, center, bar_max = map(self._get_or_create_datasource, data)
+            self.value_range.add(bar_min, center, bar_max)
+            min = None
+            max = None
+        elif len(data) == 5:
+            index, min, bar_min, bar_max, max = \
+                            map(self._get_or_create_datasource, data)
+            self.value_range.add(min, bar_min, bar_max, max)
+            center = None
+        elif len(data) == 6:
+            index, min, bar_min, center, bar_max, max = \
+                            map(self._get_or_create_datasource, data)
+            self.value_range.add(min, bar_min, center, bar_max, max)
+        self.index_range.add(index)
+
+        if styles.get("bar_color") == "auto" or styles.get("color") == "auto":
+            self._auto_color_idx = \
+                (self._auto_color_idx + 1) % len(self.auto_colors)
+            styles["color"] = self.auto_colors[self._auto_color_idx]
+
+        if self.index_scale == "linear":
+            imap = LinearMapper(range=self.index_range, 
+                        stretch_data=self.index_mapper.stretch_data)
+        else:
+            imap = LogMapper(range=self.index_range,
+                        stretch_data=self.index_mapper.stretch_data)
+        if self.value_scale == "linear":
+            vmap = LinearMapper(range=self.value_range,
+                        stretch_data=self.value_mapper.stretch_data)
+        else:
+            vmap = LogMapper(range=self.value_range,
+                        stretch_data=self.value_mapper.stretch_data)
+
+        cls = self.renderer_map["candle"]
+        plot = cls(index = index,
+                          min_values = min, 
+                          bar_min = bar_min,
+                          center_values = center,
+                          bar_max = bar_max, 
+                          max_values = max,
+                          index_mapper = imap,
+                          value_mapper = vmap,
+                          orientation = self.orientation,
+                          origin = self.origin,
+                          **styles)
+        self.add(plot)
+        self.plots[name] = [plot]
+        return [plot]
 
     def delplot(self, *names):
         """ Removes the named sub-plots. """
@@ -801,9 +932,6 @@ class Plot(DataView):
     def __title_changed(self, old, new):
         self._overlay_change_helper(old, new)
 
-    def _title_font_changed(self, old, new):
-        self._title.font = new
-
     def _legend_changed(self, old, new):
         self._overlay_change_helper(old, new)
         if new:
@@ -860,4 +988,13 @@ class Plot(DataView):
             return self._title.overlay_position
         else:
             return None
+        
+    def _set_title_font(self, font):
+        old_font = self._title.font
+        self._title.font = font
+        self.trait_property_changed("title_font", old_font, font)
+        
+    def _get_title_font(self):
+        return self._title.font
+            
 
