@@ -7,16 +7,16 @@ from numpy import abs, argmin, around, array, asarray, compress, invert, isnan,
                 sqrt, sum, transpose, where
 
 # Enthought library imports
-from enthought.enable.api import black_color_trait, ColorTrait
+from enthought.enable.api import black_color_trait, ColorTrait, AbstractMarker, \
+        CustomMarker, MarkerNameDict, MarkerTrait
 from enthought.kiva import STROKE
 from enthought.traits.api import Any, Array, Bool, Float, Int, Trait
 from enthought.traits.ui.api import View, VGroup, Item
 
 # Local relative imports
 from base_xy_plot import BaseXYPlot
-from scatter_markers import AbstractMarker, CustomMarker, \
-                            MarkerNameDict, marker_trait
 from speedups import scatterplot_gather_points
+from base import reverse_map_1d
 
 
 class ScatterPlotView(View):
@@ -49,7 +49,7 @@ class ScatterPlot(BaseXYPlot):
 
     # The type of marker to use.  This is a mapped trait using strings as the
     # keys.
-    marker = marker_trait
+    marker = MarkerTrait
 
     # The pixel size of the marker, not including the thickness of the outline.
     marker_size = Float(4.0)
@@ -75,7 +75,7 @@ class ScatterPlot(BaseXYPlot):
     # datasource.
     #------------------------------------------------------------------------
 
-    selection_marker = marker_trait
+    selection_marker = MarkerTrait
 
     selection_marker_size = Float(4.0)
 
@@ -101,17 +101,17 @@ class ScatterPlot(BaseXYPlot):
 
     def map_screen(self, data_array):
         """ Maps an array of data points into screen space and returns it as
-        an array. 
-        
+        an array.
+
         Implements the AbstractPlotRenderer interface.
         """
         # data_array is Nx2 array
         if len(data_array) == 0:
             return []
-        
+
         # XXX: For some reason, doing the tuple unpacking doesn't work:
         #        x_ary, y_ary = transpose(data_array)
-        # There is a mysterious error "object of too small depth for 
+        # There is a mysterious error "object of too small depth for
         # desired array".  However, if you catch this exception and
         # try to execute the very same line of code again, it works
         # without any complaints.
@@ -134,7 +134,7 @@ class ScatterPlot(BaseXYPlot):
 
     def map_data(self, screen_pt, all_values=True):
         """ Maps a screen space point into the "index" space of the plot.
-        
+
         Overrides the BaseXYPlot implementation, and always returns an
         array of (index, value) tuples.
         """
@@ -147,24 +147,65 @@ class ScatterPlot(BaseXYPlot):
     def map_index(self, screen_pt, threshold=0.0, outside_returns_none=True, \
                   index_only = False):
         """ Maps a screen space point to an index into the plot's index array(s).
-        
+
         Overrides the BaseXYPlot implementation..
         """
-        # Brute force implementation
-        all_data = transpose(array([self.index.get_data(), self.value.get_data()]))
-        screen_points = around(self.map_screen(all_data))
-        if len(screen_points) == 0:
-            return None
-        if index_only:
-            distances = abs(screen_points[:,0] - screen_pt[0])
+        if index_only and self.index.sort_order != "none":
+            data_pt = self.map_data(screen_pt)[0]
+            # The rest of this was copied out of BaseXYPlot.
+            # We can't just used BaseXYPlot.map_index because
+            # it expect map_data to return a value, not a pair.
+            if ((data_pt < self.index_mapper.range.low) or \
+                (data_pt > self.index_mapper.range.high)) and outside_returns_none:
+                return None
+            index_data = self.index.get_data()
+            value_data = self.value.get_data()
+
+            if len(value_data) == 0 or len(index_data) == 0:
+                return None
+
+            try:
+                ndx = reverse_map_1d(index_data, data_pt, self.index.sort_order)
+            except IndexError, e:
+                # if reverse_map raises this exception, it means that data_pt is
+                # outside the range of values in index_data.
+                if outside_returns_none:
+                    return None
+                else:
+                    if data_pt < index_data[0]:
+                        return 0
+                    else:
+                        return len(index_data) - 1
+
+            if threshold == 0.0:
+                # Don't do any threshold testing
+                return ndx
+
+            x = index_data[ndx]
+            y = value_data[ndx]
+            if isnan(x) or isnan(y):
+                return None
+            sx, sy = self.map_screen([x,y])
+            if ((threshold == 0.0) or (screen_pt[0]-sx) < threshold):
+                return ndx
+            else:
+                return None
         else:
-            delta = screen_points - array([screen_pt])
-            distances = sqrt(sum(delta*delta, axis=1))
-        closest_ndx = argmin(distances)
-        if distances[closest_ndx] <= threshold:
-            return closest_ndx
-        else:
-            return None
+            # Brute force implementation
+            all_data = transpose(array([self.index.get_data(), self.value.get_data()]))
+            screen_points = around(self.map_screen(all_data))
+            if len(screen_points) == 0:
+                return None
+            if index_only:
+                distances = abs(screen_points[:,0] - screen_pt[0])
+            else:
+                delta = screen_points - array([screen_pt])
+                distances = sqrt(sum(delta*delta, axis=1))
+            closest_ndx = argmin(distances)
+            if distances[closest_ndx] <= threshold:
+                return closest_ndx
+            else:
+                return None
 
 
     #------------------------------------------------------------------------
@@ -173,7 +214,7 @@ class ScatterPlot(BaseXYPlot):
 
     def _gather_points_old(self):
         """
-        Collects the data points that are within the bounds of the plot and 
+        Collects the data points that are within the bounds of the plot and
         caches them
         """
         if self._cache_valid and self._selection_cache_valid:
@@ -230,7 +271,7 @@ class ScatterPlot(BaseXYPlot):
                         continue
                 else:
                     continue
-                
+
                 self._cached_selection_point_mask = point_mask
                 self._cached_selected_pts = points
                 self._selection_cache_valid = True
@@ -264,7 +305,7 @@ class ScatterPlot(BaseXYPlot):
 
         points, selections = scatterplot_gather_points(index, index_range.low, index_range.high,
                                     value, value_range.low, value_range.high,
-                                    index_mask = index_mask, 
+                                    index_mask = index_mask,
                                     value_mask = value_mask,
                                     **kw)
 
@@ -280,7 +321,7 @@ class ScatterPlot(BaseXYPlot):
                 self._cached_selected_pts = None
                 self._selection_cache_valid = True
 
-        
+
     def _gather_points(self):
         #self._gather_points_fast()
         self._gather_points_old()
@@ -361,10 +402,10 @@ def render_markers(gc, points, marker, marker_size,
                    color, line_width, outline_color,
                    custom_symbol=None, debug=False):
     """ Helper function for a PlotComponent instance to render a
-    set of (x,y) points onto a graphics context.  Currently, it makes some 
-    assumptions about the attributes on the plot object; these may be factored 
+    set of (x,y) points onto a graphics context.  Currently, it makes some
+    assumptions about the attributes on the plot object; these may be factored
     out eventually.
-    
+
     Parameters
     ----------
     gc : GraphicsContext
@@ -379,7 +420,7 @@ def render_markers(gc, points, marker, marker_size,
         The color of the markers
     line_width : number
         The width, in pixels, of the marker outline
-    outline_color : RGB(A) color 
+    outline_color : RGB(A) color
         The color of the marker outline
     custom_symbol : CompiledPath
         If the marker style is 'custom', this is the symbol
@@ -398,6 +439,10 @@ def render_markers(gc, points, marker, marker_size,
 
     gc.set_line_dash(None)
     if marker.draw_mode == STROKE:
+        # markers with the STROKE draw mode will not be visible
+        # if the line width is zero, so set it to 1
+        if line_width == 0:
+            line_width = 1.0
         gc.set_stroke_color(color)
         gc.set_line_width(line_width)
     else:
