@@ -14,14 +14,14 @@ from tool_history_mixin import ToolHistoryMixin
 
 class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     """ Selects a range along the index or value axis.
-    
-    The user left-click-drags to select a region to zoom in. 
+
+    The user left-click-drags to select a region to zoom in.
     Certain keyboard keys are mapped to performing zoom actions as well.
-    
+
     Implements a basic "zoom stack" so the user move go backwards and forwards
     through previous zoom regions.
     """
-    
+
     # The selection mode:
     #
     # range:
@@ -29,58 +29,62 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     # box:
     #   Perform a "box" selection on two axes.
     tool_mode = Enum("box", "range")
-    
+
     # Is the tool always "on"? If True, left-clicking always initiates
     # a zoom operation; if False, the user must press a key to enter zoom mode.
     always_on = Bool(False)
-    
+
+    # Defines a meta-key, that works with always_on to set the zoom mode. This
+    # is useful when the zoom tool is used in conjunction with the pan tool.
+    always_on_modifier = Enum(None, 'shift', 'control', 'alt')
+
     #-------------------------------------------------------------------------
     # Zoom control
     #-------------------------------------------------------------------------
-    
+
     # The axis to which the selection made by this tool is perpendicular. This
     # only applies in 'range' mode.
     axis = Enum("index", "value")
-    
+
     #-------------------------------------------------------------------------
     # Interaction control
     #-------------------------------------------------------------------------
-    
+
     # Enable the mousewheel for zooming?
     enable_wheel = Bool(True)
-    
+
     # The mouse button that initiates the drag.  If "None", then the tool
     # will not respond to drag.  (It can still respond to mousewheel events.)
     drag_button = Enum("left", "right", None)
-    
+
     # Conversion ratio from wheel steps to zoom factors.
     wheel_zoom_step = Float(1.0)
-    
+
     # The key press to enter zoom mode, if **always_on** is False.  Has no effect
     # if **always_on** is True.
     enter_zoom_key = Instance(KeySpec, args=("z",))
-    
+
     # The key press to leave zoom mode, if **always_on** is False.  Has no effect
     # if **always_on** is True.
     exit_zoom_key = Instance(KeySpec, args=("z",))
-    
+
     # Disable the tool after the zoom is completed?
     disable_on_complete = Bool(True)
-    
+
     # The minimum amount of screen space the user must select in order for
     # the tool to actually take effect.
     minimum_screen_delta = Int(10)
-    
+
     #-------------------------------------------------------------------------
     # Appearance properties (for Box mode)
     #-------------------------------------------------------------------------
-    
+
     # The pointer to use when drawing a zoom box.
     pointer = "magnifier"
-    
+
     # The color of the selection box.
     color = ColorTrait("lightskyblue")
-    
+
     # The alpha value to apply to **color** when filling in the selection
     # region.  Because it is almost certainly useless to have an opaque zoom
     # rectangle, but it's also extremely useful to be able to use the normal
@@ -88,20 +92,20 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     # separate alpha value that replaces the alpha value of **color** at draw
     # time.
     alpha = Trait(0.4, None, Float)
-    
+
     # The color of the outside selection rectangle.
     border_color = ColorTrait("dodgerblue")
-    
+
     # The thickness of selection rectangle border.
     border_size = Int(1)
-    
+
     # The possible event states of this zoom tool.
     event_state = Enum("normal", "selecting")
-       
+
     #------------------------------------------------------------------------
     # Key mappings
     #------------------------------------------------------------------------
-    
+
     # The key that cancels the zoom and resets the view to the original defaults.
     cancel_zoom_key = Instance(KeySpec, args=("Esc",))
 
@@ -146,8 +150,8 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
 
     def enable(self, event=None):
         """ Provides a programmatic way to enable this tool, if
-        **always_on** is False. 
-        
+        **always_on** is False.
+
         Calling this method has the same effect as if the user pressed the
         **enter_zoom_key**.
         """
@@ -160,8 +164,8 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
 
     def disable(self, event=None):
         """ Provides a programmatic way to enable this tool, if **always_on**
-        is False. 
-        
+        is False.
+
         Calling this method has the same effect as if the user pressed the
         **exit_zoom_key**.
         """
@@ -188,7 +192,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
 
     def overlay(self, component, gc, view_bounds=None, mode="normal"):
         """ Draws this component overlaid on another component.
-        
+
         Overrides AbstractOverlay.
         """
         if self.event_state == "selecting":
@@ -240,7 +244,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
         lower_left[1-axis_ndx] = self.component.position[1-axis_ndx]
         upper_right[axis_ndx] = self._screen_end[axis_ndx] - self._screen_start[axis_ndx]
         upper_right[1-axis_ndx] = self.component.bounds[1-axis_ndx]
-       
+
         gc.save_state()
         try:
             gc.set_antialias(0)
@@ -257,30 +261,30 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     def normal_left_down(self, event):
         """ Handles the left mouse button being pressed while the tool is
         in the 'normal' state.
-        
+
         If the tool is enabled or always on, it starts selecting.
         """
-        if self.always_on or self._enabled:
-            # we need to make sure that there isn't another active tool that we will
-            # interfere with.
-            if self.drag_button == "left":
-                self._start_select(event)
+        if self._is_enabling_event(event):
+            self._start_select(event)
+            event.handled = True
+
         return
 
     def normal_right_down(self, event):
         """ Handles the right mouse button being pressed while the tool is
         in the 'normal' state.
-        
+
         If the tool is enabled or always on, it starts selecting.
         """
-        if self.always_on or self._enabled:
-            if self.drag_button == "right":
-                self._start_select(event)
+        if self._is_enabling_event(event):
+            self._start_select(event)
+            event.handled = True
+
         return
 
     def selecting_mouse_move(self, event):
         """ Handles the mouse moving when the tool is in the 'selecting' state.
-        
+
         The selection is extended to the current mouse position.
         """
         self._screen_end = (event.x, event.y)
@@ -291,7 +295,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     def selecting_left_up(self, event):
         """ Handles the left mouse button being released when the tool is in
         the 'selecting' state.
-        
+
         Finishes selecting and does the zoom.
         """
         if self.drag_button == "left":
@@ -301,7 +305,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     def selecting_right_up(self, event):
         """ Handles the right mouse button being released when the tool is in
         the 'selecting' state.
-        
+
         Finishes selecting and does the zoom.
         """
         if self.drag_button == "right":
@@ -311,7 +315,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     def selecting_mouse_leave(self, event):
         """ Handles the mouse leaving the plot when the tool is in the
         'selecting' state.
-        
+
         Ends the selection operation without zooming.
         """
         self._end_selecting(event)
@@ -320,7 +324,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     def selecting_key_pressed(self, event):
         """ Handles a key being pressed when the tool is in the 'selecting'
         state.
-        
+
         If the key pressed is the **cancel_zoom_key**, then selecting is
         canceled.
         """
@@ -349,26 +353,26 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
         the zoom stack, and does the zoom.
         """
         self._screen_end = (event.x, event.y)
-    
+
         start = array(self._screen_start)
         end = array(self._screen_end)
-       
+
         if sum(abs(end - start)) < self.minimum_screen_delta:
             self._end_selecting(event)
             event.handled = True
             return
-       
+
         if self.tool_mode == "range":
             mapper = self._get_mapper()
             axis = self._determine_axis()
             low = mapper.map_data(self._screen_start[axis])
             high = mapper.map_data(self._screen_end[axis])
-           
+
             if low > high:
                 low, high = high, low
         else:
             low, high = self._map_coordinate_box(self._screen_start, self._screen_end)
-    
+
         new_zoom_range = (low, high)
         self._append_state(new_zoom_range)
         self._do_zoom()
@@ -394,7 +398,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
         # Sets the bounds on the component using _cur_stack_index
         low, high = self._current_state()
         orig_low, orig_high = self._history[0]
-    
+
         if self._history_index == 0:
             if self.tool_mode == "range":
                 mapper = self._get_mapper()
@@ -411,8 +415,8 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
                 # resetting the ranges will allow 'auto' to pick the values
                 x_range.reset()
                 y_range.reset()
-               
-        else:   
+
+        else:
             if self.tool_mode == "range":
                 mapper = self._get_mapper()
                 if self._zoom_limit_reached(orig_low, orig_high, low, high, mapper):
@@ -433,13 +437,13 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
                 y_range = self.component.y_mapper.range
                 x_range.low, y_range.low = low
                 x_range.high, y_range.high = high
-                
+
         self.component.request_redraw()
         return
 
     def normal_key_pressed(self, event):
         """ Handles a key being pressed when the tool is in 'normal' state.
-        
+
         If the tool is not always on, this method handles turning it on and
         off when the appropriate keys are pressed. Also handles keys to
         manipulate the tool history.
@@ -457,9 +461,9 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
                 self._enabled = False
                 event.window.set_pointer("arrow")
                 return
-           
+
         self._history_handle_key(event)
-       
+
         if event.handled:
             self.component.request_redraw()
         return
@@ -467,7 +471,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     def normal_mouse_wheel(self, event):
         """ Handles the mouse wheel being used when the tool is in the 'normal'
         state.
-        
+
         Scrolling the wheel "up" zooms in; scrolling it "down" zooms out.
         """
         if self.enable_wheel and event.mouse_wheel != 0:
@@ -477,20 +481,20 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
             elif event.mouse_wheel < 0:
                 # zoom out
                 zoom = 1.0 + 0.5 * self.wheel_zoom_step
-           
+
             # We'll determine the current position of the cursor in screen coordinates,
             # and only afterwards map to dataspace.
             c = self.component
             screenlow_pt, screenhigh_pt = (c.x, c.y), (c.x2, c.y2)
             mouse_pos = (event.x, event.y)
-    
+
             if self.tool_mode == "range":
                 mapper_list = [(self._determine_axis(), self._get_mapper())]
             else:
                 mapper_list = [(0, c.x_mapper), (1, c.y_mapper)]
-           
+
             orig_low, orig_high = self._history[0]
-           
+
             # If any of the axes reaches its zoom limit, we should cancel the zoom.
             # We should first calculate the new ranges and store them. If none of
             # the axes reach zoom limit, we can apply the new ranges.
@@ -500,15 +504,15 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
                 mouse_val = mouse_pos[ndx]
                 newscreenlow = mouse_val + zoom * (screenlow_pt[ndx] - mouse_val)
                 newscreenhigh = mouse_val + zoom * (screenhigh_pt[ndx] - mouse_val)
-               
+
                 newlow = mapper.map_data(newscreenlow)
                 newhigh = mapper.map_data(newscreenhigh)
-                               
+
                 if type(orig_high) in (tuple,list):
                     ol, oh = orig_low[ndx], orig_high[ndx]
                 else:
                     ol, oh = orig_low, orig_high
-               
+
                 if self._zoom_limit_reached(ol, oh, newlow, newhigh, mapper):
                     # Ignore other axes, we're done.
                     event.handled = True
@@ -516,7 +520,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
                 todo_list.append((mapper,newlow,newhigh))
 
             # Check the domain limits on each dimension, and rescale the zoom
-            # amount if necessary. 
+            # amount if necessary.
             for ndx, (mapper, newlow, newhigh) in enumerate(todo_list):
                 domain_min, domain_max = getattr(mapper, "domain_limits", (None,None))
                 if domain_min is not None and newlow < domain_min:
@@ -530,16 +534,41 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
                 if newlow > newhigh:
                     newlow, newhigh = newhigh, newlow
                 mapper.range.set_bounds(newlow, newhigh)
-    
+
             event.handled = True
             c.request_redraw()
         return
 
+    def _is_enabling_event(self, event):
+        always_on = self.always_on
+        if self.always_on_modifier == 'shift':
+            always_on = always_on and event.shift_down
+        elif self.always_on_modifier == 'control':
+            always_on = always_on and event.control_down
+        elif self.always_on_modifier == 'alt':
+            always_on = always_on and event.alt_down
+
+        if always_on or self._enabled:
+            if event.right_down and self.drag_button == 'right':
+                return True
+            if event.left_down and self.drag_button == 'left':
+                return True
+
+        return False
+
     def _component_changed(self):
         if self._get_mapper() is not None:
             self._reset_state_to_current()
         return
 
+    def _tool_mode_changed(self, old, new):
+        if not self.traits_inited():
+            return
+
+        # The history must be reset because the different tool modes keep
+        # different state types in the history
+        self._reset_state_to_current()
+
     #------------------------------------------------------------------------
     # Implementation of PlotComponent interface
     #------------------------------------------------------------------------
@@ -548,7 +577,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
         """ Called by PlotComponent to set this as the active tool.
         """
         self.enable()
-    
+
     #------------------------------------------------------------------------
     # implementations of abstract methods on ToolHistoryMixin
     #------------------------------------------------------------------------
@@ -583,14 +612,14 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
                                (xhigh, yhigh)))
 
     def _reset_state_pressed(self):
-        """ Called when the tool needs to reset its history. 
-        
+        """ Called when the tool needs to reset its history.
+
         The history index will have already been set to 0. Implements
         ToolHistoryMixin.
         """
         # First zoom to the set state (ZoomTool handles setting the index=0).
         self._do_zoom()
-    
+
         # Now reset the state to the current bounds settings.
         self._reset_state_to_current()
         return
@@ -598,7 +627,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     def _prev_state_pressed(self):
         """ Called when the tool needs to advance to the previous state in the
         stack.
-        
+
         The history index will have already been set to the index corresponding
         to the prev state. Implements ToolHistoryMixin.
         """
@@ -607,7 +636,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
 
     def _next_state_pressed(self):
         """ Called when the tool needs to advance to the next state in the stack.
-        
+
         The history index will have already been set to the index corresponding
         to the next state. Implements ToolHistoryMixin.
         """
@@ -619,6 +648,7 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
     def __getstate__(self):
         dont_pickle = [
             'always_on',
+            'always_on_modifier',
             'enter_zoom_key',
             'exit_zoom_key',
             'minimum_screen_delta',
@@ -636,4 +666,4 @@ class SimpleZoom(AbstractOverlay, ToolHistoryMixin, BaseZoomTool):
                 del state[key]
 
         return state
-    
+
