@@ -23,6 +23,7 @@ from enthought.traits.ui.extras.demo import demo
 # Uncomment the config_filename portion to see a tree editor based on the
 # examples.cfg file.
 demo(use_files=True, 
-     config_filename='examples.cfg'
+     config_filename='examples.cfg',
+     title = 'Chaco Demos'
     )
     
